"use client";

import * as React from "react";

import Link from "next/link";

import { motion } from "framer-motion";

import { useLogin, usePrivy } from "@privy-io/react-auth";

import { z } from "zod";
import { useForm, UseFormReturn } from "react-hook-form";
import { zodResolver } from "@hookform/resolvers/zod";

import { useMutation, useQueryClient } from "@tanstack/react-query";

import axios from "axios";

import { Braces, Check, Code, MessageCircle } from "lucide-react";

import { toast } from "sonner";
import confetti from "canvas-confetti";
import { defineStepper, Stepper } from "@stepperize/react";
import {
  AnimatedSpan,
  Terminal,
  TypingAnimation,
} from "@/components/ui/terminal";

import { createAgentSchema } from "../schemas/create-agent-schema";

import { cn } from "@/lib/utils";
import { ANIMATION_EASE } from "@/lib/config";
import { Tables } from "@/utils/supabase/database.types";
import { createClient } from "@/utils/supabase/client";

import { Button, buttonVariants } from "@/components/ui/button";
import { Separator } from "@/components/ui/separator";

// import CardBackground from "@/components/card-background";
import AgentForm from "./agent-form";

import Spinner from "@/components/spinner";

const data = [
  {
    id: "agentType",
    title: "Agent Type",
    description: "Select how your Agent will interact",
  },
  {
    id: "agentForm",
    title: "Agent Form",
    description: "Enter your agent informations",
  },
  {
    id: "confirmation",
    title: "Form Confirmation",
    description: "Enter your agent informations",
  },
  { id: "complete", title: "Complete", description: "Checkout complete" },
];

const { useStepper, steps, utils } = defineStepper(...data);

const triggerConfetti = () => {
  const duration = 5 * 1000;
  const animationEnd = Date.now() + duration;
  const defaults = { startVelocity: 30, spread: 360, ticks: 60, zIndex: 0 };

  const randomInRange = (min: number, max: number) =>
    Math.random() * (max - min) + min;

  const interval = window.setInterval(() => {
    const timeLeft = animationEnd - Date.now();

    if (timeLeft <= 0) {
      return clearInterval(interval);
    }

    const particleCount = 50 * (timeLeft / duration);
    confetti({
      ...defaults,
      particleCount,
      origin: { x: randomInRange(0.1, 0.3), y: Math.random() - 0.2 },
    });
    confetti({
      ...defaults,
      particleCount,
      origin: { x: randomInRange(0.7, 0.9), y: Math.random() - 0.2 },
    });
  }, 250);
};

export function FormStepper() {
  const [agentId, setAgentId] = React.useState("");
  const [agentName, setAgenName] = React.useState("");

  const { ready, authenticated, logout, user } = usePrivy();

  const stepper = useStepper();

  const currentIndex = utils.getIndex(stepper.current.id);

  const form = useForm<z.infer<typeof createAgentSchema>>({
    resolver: zodResolver(createAgentSchema),
    defaultValues: {
      userId: "",
      name: "",
      description: "",
      modelType: "general",
    },
    mode: "onChange",
  });

  const queryClient = useQueryClient();

  const createAgentMutation = useMutation({
    mutationFn: async (values: z.infer<typeof createAgentSchema>) => {
      const { modelType, userId, ...rest } = values;

      const supabase = createClient();

      const url = encodeURI(
        `https://ai-endpoint-one.dev3vds1.link/ai-image/${values.description}`
      );

      const json = await axios.get(url);

      const response = await supabase
        .from("agents")
        .insert({
          ...rest,
          model_type: "general",
          image_url: json.data.url,
          user_id: user!.id,
          conversation: 0,
          last_conv: null,
        })
        .select()
        .limit(1)
        .single();

      if (response.error) throw new Error(response.error.message);

      return response.data;
    },
    onSuccess: (data: Tables<"agents">) => {
      queryClient.invalidateQueries({
        queryKey: ["agents"],
      });
      triggerConfetti();
      stepper.next();

      setAgentId(data.id.toString());
      setAgenName(data.name);

      stepper.next();
    },
    onError: (err) => {
      toast.error(err.message);
    },
  });

  const handleSubmit = () => {
    if (!user) {
      toast.error("You are not login yet");
      return;
    }
    const values = form.getValues();
    createAgentMutation.mutate(values);
  };

  return (
    <section>
      <div className="layout border-x px-3 sm:px-6 py-8">
        <motion.div
          initial={{ filter: "blur(10px)", opacity: 0, y: 50 }}
          animate={{ filter: "blur(0px)", opacity: 1, y: 0 }}
          transition={{
            duration: 1,
            delay: 0.5,
            ease: ANIMATION_EASE,
          }}
        >
          {/* form title  */}
          <div className="flex justify-between mb-8">
            <div>
              <h2 className="text-primary text-lg font-bold mb-1">
                Agent Form
              </h2>
              <p className="text-sm text-muted-foreground">
                {stepper.current.description}
              </p>
            </div>
          </div>
          {/* form nav  */}
          <nav aria-label="Checkout Steps" className="group mb-8">
            <ol
              className="flex items-center justify-between gap-2"
              aria-orientation="horizontal"
            >
              {stepper.all.map((step, index, array) => (
                <React.Fragment key={step.id}>
                  <li className="flex items-center gap-4 flex-shrink-0">
                    <Button
                      type="button"
                      role="tab"
                      variant={index <= currentIndex ? "default" : "outline"}
                      aria-current={
                        stepper.current.id === step.id ? "step" : undefined
                      }
                      aria-posinset={index + 1}
                      aria-setsize={steps.length}
                      aria-selected={stepper.current.id === step.id}
                      className={cn(
                        "flex size-8 items-center justify-center rounded-full",
                        index > currentIndex && "border border-dashed"
                      )}
                    >
                      {index + 1}
                    </Button>
                    <span className="text-sm font-bold">{step.title}</span>
                  </li>
                  {index < array.length - 1 && (
                    <Separator
                      className={`border-t border-dashed flex-1 ${
                        index < currentIndex ? "bg-primary" : "bg-muted"
                      }`}
                    />
                  )}
                </React.Fragment>
              ))}
            </ol>
          </nav>
          {/* form content */}
          <div className="space-y-8">
            {stepper.switch({
              agentType: () => <AgentTypeComponent stepper={stepper} />,
              agentForm: () => (
                <AgentFormComponent
                  stepper={stepper}
                  form={form}
                  isPending={createAgentMutation.isPending}
                  // onSubmit={handleSubmit}
                  onSubmit={() => stepper.next()}
                />
              ),
              confirmation: () => (
                <AgentConfirmationComponent
                  stepper={stepper}
                  form={form}
                  isPending={createAgentMutation.isPending}
                  onSubmit={handleSubmit}
                />
              ),
              complete: () => (
                <CompleteComponent agentId={agentId} agentName={agentName} />
              ),
            })}
          </div>
        </motion.div>
      </div>
    </section>
  );
}

const AgentTypeComponent = ({
  stepper,
}: {
  stepper: Stepper<
    {
      id: string;
      title: string;
      description: string;
    }[]
  >;
}) => {
  const agentTypes = [
    {
      icon: (
        <svg
          width="65"
          height="56"
          viewBox="0 0 65 56"
          fill="none"
          xmlns="http://www.w3.org/2000/svg"
        >
          <path
            d="M60 0.5H5C3.67392 0.5 2.40215 1.02678 1.46447 1.96447C0.526784 2.90215 0 4.17392 0 5.5V50.5C0 51.8261 0.526784 53.0979 1.46447 54.0355C2.40215 54.9732 3.67392 55.5 5 55.5H60C61.3261 55.5 62.5979 54.9732 63.5355 54.0355C64.4732 53.0979 65 51.8261 65 50.5V5.5C65 4.17392 64.4732 2.90215 63.5355 1.96447C62.5979 1.02678 61.3261 0.5 60 0.5ZM24.2688 38.7312C24.501 38.9635 24.6853 39.2393 24.811 39.5428C24.9367 39.8462 25.0014 40.1715 25.0014 40.5C25.0014 40.8285 24.9367 41.1538 24.811 41.4572C24.6853 41.7607 24.501 42.0365 24.2688 42.2688C24.0365 42.501 23.7607 42.6853 23.4572 42.811C23.1538 42.9367 22.8285 43.0014 22.5 43.0014C22.1715 43.0014 21.8462 42.9367 21.5428 42.811C21.2393 42.6853 20.9635 42.501 20.7312 42.2688L8.23125 29.7688C7.99881 29.5366 7.81441 29.2608 7.6886 28.9574C7.56279 28.6539 7.49803 28.3285 7.49803 28C7.49803 27.6715 7.56279 27.3461 7.6886 27.0426C7.81441 26.7392 7.99881 26.4634 8.23125 26.2312L20.7312 13.7312C21.2003 13.2621 21.8366 12.9986 22.5 12.9986C23.1634 12.9986 23.7997 13.2621 24.2688 13.7312C24.7379 14.2004 25.0014 14.8366 25.0014 15.5C25.0014 16.1634 24.7379 16.7997 24.2688 17.2688L13.5344 28L24.2688 38.7312ZM56.7688 29.7688L44.2688 42.2688C43.7996 42.7379 43.1634 43.0014 42.5 43.0014C41.8366 43.0014 41.2004 42.7379 40.7312 42.2688C40.2621 41.7996 39.9986 41.1634 39.9986 40.5C39.9986 39.8366 40.2621 39.2004 40.7312 38.7312L51.4656 28L40.7312 17.2688C40.2621 16.7997 39.9986 16.1634 39.9986 15.5C39.9986 14.8366 40.2621 14.2004 40.7312 13.7312C41.2004 13.2621 41.8366 12.9986 42.5 12.9986C43.1634 12.9986 43.7996 13.2621 44.2688 13.7312L56.7688 26.2312C57.0012 26.4634 57.1856 26.7392 57.3114 27.0426C57.4372 27.3461 57.502 27.6715 57.502 28C57.502 28.3285 57.4372 28.6539 57.3114 28.9574C57.1856 29.2608 57.0012 29.5366 56.7688 29.7688Z"
            fill="#222222"
          />
        </svg>
      ),
      heading: "Chat",
      description: "Access from our website",
    },
    {
      icon: (
        <svg
          width="65"
          height="56"
          viewBox="0 0 65 56"
          fill="none"
          xmlns="http://www.w3.org/2000/svg"
        >
          <path
            d="M60 0.5H5C3.67392 0.5 2.40215 1.02678 1.46447 1.96447C0.526784 2.90215 0 4.17392 0 5.5V50.5C0 51.8261 0.526784 53.0979 1.46447 54.0355C2.40215 54.9732 3.67392 55.5 5 55.5H60C61.3261 55.5 62.5979 54.9732 63.5355 54.0355C64.4732 53.0979 65 51.8261 65 50.5V5.5C65 4.17392 64.4732 2.90215 63.5355 1.96447C62.5979 1.02678 61.3261 0.5 60 0.5ZM24.2688 38.7312C24.501 38.9635 24.6853 39.2393 24.811 39.5428C24.9367 39.8462 25.0014 40.1715 25.0014 40.5C25.0014 40.8285 24.9367 41.1538 24.811 41.4572C24.6853 41.7607 24.501 42.0365 24.2688 42.2688C24.0365 42.501 23.7607 42.6853 23.4572 42.811C23.1538 42.9367 22.8285 43.0014 22.5 43.0014C22.1715 43.0014 21.8462 42.9367 21.5428 42.811C21.2393 42.6853 20.9635 42.501 20.7312 42.2688L8.23125 29.7688C7.99881 29.5366 7.81441 29.2608 7.6886 28.9574C7.56279 28.6539 7.49803 28.3285 7.49803 28C7.49803 27.6715 7.56279 27.3461 7.6886 27.0426C7.81441 26.7392 7.99881 26.4634 8.23125 26.2312L20.7312 13.7312C21.2003 13.2621 21.8366 12.9986 22.5 12.9986C23.1634 12.9986 23.7997 13.2621 24.2688 13.7312C24.7379 14.2004 25.0014 14.8366 25.0014 15.5C25.0014 16.1634 24.7379 16.7997 24.2688 17.2688L13.5344 28L24.2688 38.7312ZM56.7688 29.7688L44.2688 42.2688C43.7996 42.7379 43.1634 43.0014 42.5 43.0014C41.8366 43.0014 41.2004 42.7379 40.7312 42.2688C40.2621 41.7996 39.9986 41.1634 39.9986 40.5C39.9986 39.8366 40.2621 39.2004 40.7312 38.7312L51.4656 28L40.7312 17.2688C40.2621 16.7997 39.9986 16.1634 39.9986 15.5C39.9986 14.8366 40.2621 14.2004 40.7312 13.7312C41.2004 13.2621 41.8366 12.9986 42.5 12.9986C43.1634 12.9986 43.7996 13.2621 44.2688 13.7312L56.7688 26.2312C57.0012 26.4634 57.1856 26.7392 57.3114 27.0426C57.4372 27.3461 57.502 27.6715 57.502 28C57.502 28.3285 57.4372 28.6539 57.3114 28.9574C57.1856 29.2608 57.0012 29.5366 56.7688 29.7688Z"
            fill="#222222"
          />
        </svg>
      ),
      heading: "CLI",
      description: "Access from terminal",
    },
    {
      icon: (
        <svg
          width="60"
          height="66"
          viewBox="0 0 60 66"
          fill="none"
          xmlns="http://www.w3.org/2000/svg"
        >
          <path
            d="M30 0.5C13.1781 0.5 0 8.1875 0 18V48C0 57.8125 13.1781 65.5 30 65.5C46.8219 65.5 60 57.8125 60 48V18C60 8.1875 46.8219 0.5 30 0.5ZM55 33C55 36.0062 52.5375 39.0719 48.2469 41.4125C43.4156 44.0469 36.9344 45.5 30 45.5C23.0656 45.5 16.5844 44.0469 11.7531 41.4125C7.4625 39.0719 5 36.0062 5 33V27.8C10.3313 32.4875 19.4469 35.5 30 35.5C40.5531 35.5 49.6688 32.475 55 27.8V33ZM48.2469 56.4125C43.4156 59.0469 36.9344 60.5 30 60.5C23.0656 60.5 16.5844 59.0469 11.7531 56.4125C7.4625 54.0719 5 51.0062 5 48V42.8C10.3313 47.4875 19.4469 50.5 30 50.5C40.5531 50.5 49.6688 47.475 55 42.8V48C55 51.0062 52.5375 54.0719 48.2469 56.4125Z"
            fill="#222222"
          />
        </svg>
      ),
      heading: "API",
      description: "Access with API",
    },
  ];

  return (
    <>
      <div className="grid grid-cols-1 lg:grid-cols-3 gap-4">
        {agentTypes.map((agentType, idx) => (
          <Button
            disabled={idx !== 0}
            key={idx}
            variant={"outline"}
            className={cn(
              idx === 0 &&
                "bg-background rounded-xl transition-all duration-300 border shadow-[0_0_23.1px_0_rgba(0,0,0,0.08)_inset]",
              "py-14 px-6 border-2"
            )}
          >
            <div className="flex items-center gap-3">
              {agentType.icon}
              <div className="space-y-3 text-left">
                <p className="text-xl font-bold">{agentType.heading}</p>
                <p className="text-muted-foreground">{agentType.description}</p>
              </div>
            </div>
          </Button>
        ))}
      </div>
      {/* form footer  */}
      <div>
        <div className="flex justify-end gap-4">
          <Button variant="outline" disabled={true}>
            Back
          </Button>
          <Button onClick={stepper.next}>Next</Button>
        </div>
      </div>
    </>
  );
};

const AgentFormComponent = ({
  stepper,
  form,
  onSubmit,
  isPending,
}: {
  stepper: Stepper<
    {
      id: string;
      title: string;
      description: string;
    }[]
  >;
  form: UseFormReturn<z.infer<typeof createAgentSchema>, any, undefined>;
  onSubmit: (values: z.infer<typeof createAgentSchema>) => void;
  isPending: boolean;
}) => {
  return (
    <AgentForm
      stepper={stepper}
      form={form}
      onSubmit={onSubmit}
      isPending={isPending}
    />
  );
};

const AgentConfirmationComponent = ({
  stepper,
  form,
  onSubmit,
  isPending,
}: {
  stepper: Stepper<
    {
      id: string;
      title: string;
      description: string;
    }[]
  >;
  form: UseFormReturn<z.infer<typeof createAgentSchema>, any, undefined>;
  onSubmit: () => void;
  isPending: boolean;
}) => {
  return (
    <>
      <Terminal className="max-w-none max-h-none">
        <TypingAnimation duration={30}>
          &gt; This action will create your agent with following details:
        </TypingAnimation>
        <AnimatedSpan delay={2000}>
          <p>
            &gt; Agent Name:{" "}
            <span className="text-green-500">{form.getValues("name")}</span>
          </p>
        </AnimatedSpan>
        <AnimatedSpan delay={2500}>
          <p>
            &gt; Agent Desccription:{" "}
            <span className="text-green-500">
              {form.getValues("description")}
            </span>
          </p>
        </AnimatedSpan>
        <TypingAnimation delay={3000}>
          &gt; Click "Submit" to create the agent
        </TypingAnimation>
      </Terminal>
      {/* form footer  */}
      <div>
        <div className="flex justify-end gap-4">
          <Button variant="outline" onClick={stepper.prev} disabled={isPending}>
            {isPending && <Spinner className="h-4 w-4 mr-2" />}
            Back
          </Button>
          <Button onClick={() => onSubmit()} disabled={isPending}>
            {isPending && <Spinner className="h-4 w-4 mr-2" />}
            Submit
          </Button>
        </div>
      </div>
    </>
  );
};

const CompleteComponent = ({
  agentId,
  agentName,
}: {
  agentId: string;
  agentName: string;
}) => {
  return (
    <>
      <div className="flex flex-col justify-center items-center  py-4">
        <div className="h-20 w-20 flex flex-col justify-center items-center rounded-full bg-primary">
          <svg
            width="61"
            height="41"
            viewBox="0 0 61 41"
            fill="none"
            xmlns="http://www.w3.org/2000/svg"
          >
            <rect
              x="0.5"
              y="14.6426"
              width="5.85703"
              height="4.39277"
              fill="white"
            />
            <rect
              x="6.35693"
              y="19.0342"
              width="7.32129"
              height="8.78555"
              fill="white"
            />
            <rect
              x="13.6787"
              y="27.8223"
              width="5.85703"
              height="4.39277"
              fill="white"
            />
            <rect
              x="19.5354"
              y="32.2148"
              width="7.32129"
              height="8.78555"
              fill="white"
            />
            <rect
              x="26.8567"
              y="27.8223"
              width="5.85703"
              height="4.39277"
              fill="white"
            />
            <rect
              x="32.7136"
              y="19.0342"
              width="8.78555"
              height="8.78555"
              fill="white"
            />
            <rect
              x="41.4993"
              y="14.6426"
              width="5.85703"
              height="4.39277"
              fill="white"
            />
            <rect
              x="47.3562"
              y="5.85645"
              width="8.78555"
              height="8.78555"
              fill="white"
            />
            <rect x="56.1418" width="4.39277" height="5.85703" fill="white" />
          </svg>
        </div>
        <h3 className="text-xl py-4 font-bold">Agent succesfully created!</h3>
        <p className="text-lg text-muted-foreground mb-3">
          You can try to have a conversation with your agent.
        </p>
        <Link
          href={`/agents/${agentId}`}
          target="blank"
          className={buttonVariants({ variant: "default" })}
        >
          <svg
            width="16"
            height="16"
            viewBox="0 0 17 18"
            fill="none"
            xmlns="http://www.w3.org/2000/svg"
            className="mr-2"
          >
            <path
              d="M8.50006 0.875C7.0973 0.874694 5.71836 1.23757 4.49745 1.9283C3.27654 2.61904 2.25526 3.6141 1.53301 4.81664C0.810765 6.01918 0.412159 7.38822 0.375993 8.79052C0.339826 10.1928 0.66733 11.5806 1.32663 12.8188L0.439908 15.4789C0.366459 15.6992 0.355799 15.9355 0.409125 16.1615C0.462451 16.3874 0.577655 16.5941 0.741823 16.7582C0.905992 16.9224 1.11264 17.0376 1.3386 17.0909C1.56456 17.1443 1.80091 17.1336 2.02116 17.0602L4.68131 16.1734C5.77098 16.753 6.97836 17.0767 8.21181 17.12C9.44525 17.1633 10.6724 16.925 11.8 16.4232C12.9276 15.9215 13.926 15.1694 14.7196 14.2241C15.5132 13.2789 16.0809 12.1652 16.3799 10.9678C16.6788 9.77029 16.7009 8.52047 16.4447 7.31315C16.1885 6.10584 15.6605 4.97276 14.901 3.99993C14.1415 3.02711 13.1703 2.24009 12.0612 1.69864C10.9521 1.15718 9.73427 0.875506 8.50006 0.875ZM5.06256 9.9375C4.87714 9.9375 4.69589 9.88252 4.54172 9.7795C4.38755 9.67649 4.26739 9.53007 4.19643 9.35876C4.12547 9.18746 4.1069 8.99896 4.14308 8.8171C4.17925 8.63525 4.26854 8.4682 4.39965 8.33709C4.53076 8.20598 4.69781 8.11669 4.87967 8.08051C5.06152 8.04434 5.25002 8.06291 5.42133 8.13386C5.59264 8.20482 5.73905 8.32498 5.84207 8.47915C5.94508 8.63332 6.00006 8.81458 6.00006 9C6.00006 9.24864 5.90129 9.4871 5.72548 9.66291C5.54966 9.83873 5.31121 9.9375 5.06256 9.9375ZM8.50006 9.9375C8.31464 9.9375 8.13339 9.88252 7.97922 9.7795C7.82505 9.67649 7.70489 9.53007 7.63393 9.35876C7.56297 9.18746 7.5444 8.99896 7.58058 8.8171C7.61675 8.63525 7.70604 8.4682 7.83715 8.33709C7.96826 8.20598 8.13531 8.11669 8.31717 8.08051C8.49902 8.04434 8.68752 8.06291 8.85883 8.13386C9.03014 8.20482 9.17655 8.32498 9.27957 8.47915C9.38258 8.63332 9.43756 8.81458 9.43756 9C9.43756 9.24864 9.33879 9.4871 9.16298 9.66291C8.98716 9.83873 8.74871 9.9375 8.50006 9.9375ZM11.9376 9.9375C11.7521 9.9375 11.5709 9.88252 11.4167 9.7795C11.2625 9.67649 11.1424 9.53007 11.0714 9.35876C11.0005 9.18746 10.9819 8.99896 11.0181 8.8171C11.0543 8.63525 11.1435 8.4682 11.2747 8.33709C11.4058 8.20598 11.5728 8.11669 11.7547 8.08051C11.9365 8.04434 12.125 8.06291 12.2963 8.13386C12.4676 8.20482 12.6141 8.32498 12.7171 8.47915C12.8201 8.63332 12.8751 8.81458 12.8751 9C12.8751 9.24864 12.7763 9.4871 12.6005 9.66291C12.4247 9.83873 12.1862 9.9375 11.9376 9.9375Z"
              fill="white"
            />
          </svg>
          Chat with {agentName}
        </Link>
      </div>
      {/* form footer  */}
      <div>
        <div className="flex justify-end gap-4">
          <Link href="/" className={buttonVariants({ variant: "default" })}>
            Back to home
          </Link>
        </div>
      </div>
    </>
  );
};
